pub enum Operand<'ctx> {
Value(BasicValueEnum<'ctx>),
Block(BasicBlock<'ctx>),
}Expand description
Either BasicValueEnum or BasicBlock.
Variants
Value(BasicValueEnum<'ctx>)
Represents a BasicValueEnum.
Block(BasicBlock<'ctx>)
Represents a BasicBlock.
Implementations
Sourceimpl<'ctx> Operand<'ctx>
impl<'ctx> Operand<'ctx>
Sourcepub fn is_value(self) -> bool
pub fn is_value(self) -> bool
Determines if the Operand is a BasicValueEnum.
Sourcepub fn is_block(self) -> bool
pub fn is_block(self) -> bool
Determines if the Operand is a BasicBlock.
Sourcepub fn value(self) -> Option<BasicValueEnum<'ctx>>
pub fn value(self) -> Option<BasicValueEnum<'ctx>>
If the Operand is a BasicValueEnum, map it into Option::Some.
Sourcepub fn block(self) -> Option<BasicBlock<'ctx>>
pub fn block(self) -> Option<BasicBlock<'ctx>>
If the Operand is a BasicBlock, map it into Option::Some.
Sourcepub fn expect_value(self, msg: &str) -> BasicValueEnum<'ctx>
pub fn expect_value(self, msg: &str) -> BasicValueEnum<'ctx>
Expect BasicValueEnum, panic with the message if it is not.
Sourcepub fn expect_block(self, msg: &str) -> BasicBlock<'ctx>
pub fn expect_block(self, msg: &str) -> BasicBlock<'ctx>
Expect BasicBlock, panic with the message if it is not.
Sourcepub fn unwrap_value(self) -> BasicValueEnum<'ctx>
pub fn unwrap_value(self) -> BasicValueEnum<'ctx>
Unwrap BasicValueEnum. Will panic if it is not.
Sourcepub fn unwrap_block(self) -> BasicBlock<'ctx>
pub fn unwrap_block(self) -> BasicBlock<'ctx>
Unwrap BasicBlock. Will panic if it is not.