ValueKind

Enum ValueKind 

pub enum ValueKind<'ctx> {
    Basic(BasicValueEnum<'ctx>),
    Instruction(InstructionValue<'ctx>),
}
Variants



Basic(BasicValueEnum<'ctx>)

Represents a BasicValueEnum.



Instruction(InstructionValue<'ctx>)

Represents an InstructionValue.

Implementations

Source

impl<'ctx> ValueKind<'ctx>

pub fn is_basic(self) -> bool

Determines if the ValueKind is a BasicValueEnum.

pub fn is_instruction(self) -> bool

Determines if the ValueKind is an InstructionValue.

pub fn basic(self) -> Option<BasicValueEnum<'ctx>>

If the ValueKind is a BasicValueEnum, map it into Option::Some.

pub fn instruction(self) -> Option<InstructionValue<'ctx>>

If the ValueKind is an InstructionValue, map it into Option::Some.

pub fn expect_basic(self, msg: &str) -> BasicValueEnum<'ctx>

Expect BasicValueEnum, panic with the message if it is not.

pub fn expect_instruction(self, msg: &str) -> InstructionValue<'ctx>

Expect InstructionValue, panic with the message if it is not.

pub fn unwrap_basic(self) -> BasicValueEnum<'ctx>

Unwrap BasicValueEnum. Will panic if it is not.

pub fn unwrap_instruction(self) -> InstructionValue<'ctx>

Unwrap InstructionValue. Will panic if it is not.